#! python
import os
import sys

__version_info = {} # keep a reference to parse VERSION once

def get_version_info():
    # read from VERSION
    # return dictionary filled with content of version
    if not __version_info:
        this_file_dir = os.path.dirname(os.path.abspath(__file__))
        version_file = os.path.join(this_file_dir, 'VERSION')

        if not os.path.isfile(version_file):
            raise SystemError("VERSION file not found.")

        for l in open(version_file).readlines():
            if l.startswith('AUBIO_MAJOR_VERSION'):
                __version_info['AUBIO_MAJOR_VERSION'] = int(l.split('=')[1])
            if l.startswith('AUBIO_MINOR_VERSION'):
                __version_info['AUBIO_MINOR_VERSION'] = int(l.split('=')[1])
            if l.startswith('AUBIO_PATCH_VERSION'):
                __version_info['AUBIO_PATCH_VERSION'] = int(l.split('=')[1])
            if l.startswith('AUBIO_VERSION_STATUS'):
                __version_info['AUBIO_VERSION_STATUS'] = \
                    l.split('=')[1].strip()[1:-1]

            if l.startswith('LIBAUBIO_LT_CUR'):
                __version_info['LIBAUBIO_LT_CUR'] = int(l.split('=')[1])
            if l.startswith('LIBAUBIO_LT_REV'):
                __version_info['LIBAUBIO_LT_REV'] = int(l.split('=')[1])
            if l.startswith('LIBAUBIO_LT_AGE'):
                __version_info['LIBAUBIO_LT_AGE'] = int(l.split('=')[1])

        if len(__version_info) < 6:
            raise SystemError("Failed parsing VERSION file.")

        # switch version status with commit sha in alpha releases
        if __version_info['AUBIO_VERSION_STATUS'] and \
                '~alpha' in __version_info['AUBIO_VERSION_STATUS']:
            AUBIO_GIT_SHA = get_git_revision_hash()
            if AUBIO_GIT_SHA:
                __version_info['AUBIO_VERSION_STATUS'] = '~git+' + AUBIO_GIT_SHA

    return __version_info

def get_libaubio_version():
    verfmt = '%(LIBAUBIO_LT_CUR)s.%(LIBAUBIO_LT_REV)s.%(LIBAUBIO_LT_AGE)s'
    return str(verfmt % get_version_info())

def get_aubio_version():
    verfmt = '%(AUBIO_MAJOR_VERSION)s.%(AUBIO_MINOR_VERSION)s.%(AUBIO_PATCH_VERSION)s%(AUBIO_VERSION_STATUS)s'
    return str(verfmt % get_version_info())

def get_aubio_pyversion():
    # convert to version for python according to pep 440
    # see https://www.python.org/dev/peps/pep-0440/
    # outputs MAJ.MIN.PATCH[a0[+git.<sha>[.mods]]]
    aubio_version = get_aubio_version()
    if '~git+' in aubio_version:
        pep440str = aubio_version.replace('+', '.')
        verstr = pep440str.replace('~git.', 'a0+')
    elif '~alpha' in aubio_version:
        verstr = aubio_version.replace('~alpha', 'a0')
    else:
        verstr = aubio_version
    return verstr

def get_git_revision_hash(short=True):
    # get commit id, with +mods if local tree is not clean
    if not os.path.isdir('.git'):
        # print('Version : not in git repository : can\'t get sha')
        return None
    import subprocess
    aubio_dir = os.path.dirname(os.path.abspath(__file__))
    if not os.path.exists(aubio_dir):
        raise SystemError("git / root folder not found")
    gitcmd = ['git', '-C', aubio_dir, 'rev-parse']
    if short:
        gitcmd.append('--short')
    gitcmd.append('HEAD')
    try:
        gitsha = subprocess.check_output(gitcmd).strip().decode('utf8')
    except Exception as e:
        sys.stderr.write('git command error :%s\n' % e)
        return None
    # check if we have a clean tree
    gitcmd = ['git', '-C', aubio_dir, 'status', '--porcelain']
    try:
        output = subprocess.check_output(gitcmd).decode('utf8')
        if len(output):
            sys.stderr.write('Info: current tree is not clean\n\n')
            sys.stderr.write(output + '\n')
            gitsha += '+mods'
    except subprocess.CalledProcessError as e:
        sys.stderr.write('git command error :%s\n' % e)
        pass
    return gitsha

if __name__ == '__main__':
    if len(sys.argv) > 1 and sys.argv[1] == '-v':
        print (get_aubio_version())
    elif len(sys.argv) > 1 and sys.argv[1] == '-p':
        print (get_aubio_version())
    else:
        print ('%30s'% 'aubio version:', get_aubio_version())
        print ('%30s'% 'python-aubio version:', get_aubio_pyversion())



import re
import os.path
from waflib import TaskGen, Task
from waflib.Context import STDOUT
from waflib.Utils import O644

class gen_sym_file(Task.Task):
    color = 'BLUE'
    inst_to = '${LIBDIR}'
    def run(self):
        syms = {}
        reg = getattr(self.generator, 'export_symbols_regex','.+?')
        if 'msvc' in self.env.CC_NAME:
            outputs = [x.abspath() for x in self.generator.link_task.outputs]
            binary_path = list(filter(lambda x: x.endswith('lib'), outputs))[0]
            reg_compiled = re.compile(r'External\s+\|\s+(?P<symbol>%s)\b' % reg)
            cmd =(self.env.LINK_CC) + ['/dump', '/symbols', binary_path]
        else: # using gcc? assume we have nm
            outputs = [x.abspath() for x in self.generator.link_task.outputs]
            binary_path = list(filter(lambda x: x.endswith('dll'), outputs))[0]
            reg_compiled = re.compile(r'(T|D)\s+_(?P<symbol>%s)\b'%reg)
            cmd = (self.env.NM or ['nm']) + ['-g', binary_path]
        dump_output = self.generator.bld.cmd_and_log(cmd, quiet=STDOUT)
        syms = set([])
        for m in reg_compiled.finditer(dump_output):
            syms.add(m.group('symbol'))
        syms = list(syms)
        syms.sort()
        self.outputs[0].write('EXPORTS\n'+'\n'.join(syms))

@TaskGen.feature('gensyms')
@TaskGen.after_method('process_source','process_use','apply_link','process_uselib_local','propagate_uselib_vars')
def gen_symbols(self):
    #sym_file = self.path.find_or_declare(self.target + '.def')
    sym_file_name = os.path.splitext(self.link_task.outputs[0].abspath())[0] + '.def'
    sym_file = self.path.find_or_declare(sym_file_name)
    symtask = self.create_task('gen_sym_file', self.link_task.outputs, sym_file)
    self.add_install_files(install_to=self.link_task.inst_to, install_from=sym_file,
        chmod=O644, task=self.link_task)




#! /usr/bin/env python

import sys
import os.path
import glob
from setuptools import setup, Extension

# add ./python/lib to current path
sys.path.append(os.path.join('python', 'lib'))  # noqa
from moresetuptools import build_ext, CleanGenerated

# function to generate gen/*.{c,h}
from this_version import get_aubio_version, get_aubio_pyversion

__version__ = get_aubio_pyversion()
__aubio_version__ = get_aubio_version()

include_dirs = []
library_dirs = []
define_macros = [('AUBIO_VERSION', '%s' % __aubio_version__)]
extra_link_args = []

include_dirs += ['python/ext']
try:
    import numpy
    include_dirs += [numpy.get_include()]
except ImportError:
    pass

if sys.platform.startswith('darwin'):
    extra_link_args += ['-framework', 'CoreFoundation',
            '-framework', 'AudioToolbox']

sources = sorted(glob.glob(os.path.join('python', 'ext', '*.c')))

aubio_extension = Extension("aubio._aubio",
    sources,
    include_dirs = include_dirs,
    library_dirs = library_dirs,
    extra_link_args = extra_link_args,
    define_macros = define_macros)

# TODO: find a way to track if package is built against libaubio
# if os.path.isfile('src/aubio.h'):
#     if not os.path.isdir(os.path.join('build','src')):
#         pass
#         #__version__ += 'a2' # python only version

classifiers = [
    'Development Status :: 4 - Beta',
    'Environment :: Console',
    'Intended Audience :: Science/Research',
    'Topic :: Software Development :: Libraries',
    'Topic :: Multimedia :: Sound/Audio :: Analysis',
    'Topic :: Multimedia :: Sound/Audio :: Sound Synthesis',
    'Operating System :: POSIX',
    'Operating System :: MacOS :: MacOS X',
    'Operating System :: Microsoft :: Windows',
    'Programming Language :: C',
    'Programming Language :: Python',
    'License :: OSI Approved :: '
    'GNU General Public License v3 or later (GPLv3+)',
    ]

thisdir = os.path.abspath(os.path.dirname(__file__))
py_readme_file = os.path.join(thisdir, 'python', 'README.md')
with open(py_readme_file, 'r') as fp:
    long_description = ''.join(fp.readlines()[3:])

distrib = setup(name='aubio_ledfx',
    version = __version__,
    packages = ['aubio'],
    package_dir = {'aubio': 'python/lib/aubio'},
    ext_modules = [aubio_extension],
    description = 'a collection of tools for music analysis',
    long_description = long_description,
    long_description_content_type = 'text/markdown',
    license = 'GNU/GPL version 3',
    author = 'Paul Brossier',
    author_email = 'piem@aubio.org',
    maintainer = 'Paul Brossier',
    maintainer_email = 'piem@aubio.org',
    url = 'https://aubio.org/',
    platforms = 'any',
    classifiers = classifiers,
    install_requires = ['numpy'],
    setup_requires = ['numpy'],
    cmdclass = {
        'clean': CleanGenerated,
        'build_ext': build_ext,
        },
    entry_points = {
        'console_scripts': [
            'aubio = aubio.cmd:main',
            'aubiocut = aubio.cut:main',
        ],
    },
    )
